<?php

namespace Kash;

use Psr\SimpleCache\CacheInterface;

trait HandleCacheTrait
{
    protected ? CacheInterface $cache;

    protected function handleCaching(string|array $key, callable $callback, null|int|\DateInterval $ttl = null) : mixed
    {
        static $internalCache = [];

        if (is_array($key)) {
            $key = implode('.', $key);
        }

        if ( isset($internalCache[$key]) ) {
            return $internalCache[$key];
        }

        if ($this->cache !== null) {
            if (null === ($internalCache[$key] = $this->cache->get($key, null))) {
                $internalCache[$key] = call_user_func($callback);

                $this->cache->set($key, $internalCache[$key], $ttl);
            }
        }
        else {
            $internalCache[$key] = call_user_func($callback);
        }

        return $internalCache[$key];
    }
}